@@ -1,6 +1,6 @@
 #include "composite-engine.h"
 
-#define DO_TIMINGS 1
+#define DO_TIMINGS 0 		/* enable this for lowlight timings */
 
 #if DO_TIMINGS
 #include <sys/time.h>
@@ -37,6 +37,9 @@ static StackItem *comp_stack;
 
 #define stack_enumerate(c) for((c) = comp_stack; (c); (c) = (c)->next)
 
+#define stack_enumerate_backwards(c) for((c) = stack_last(); (c); (c) = stack_prev((c)->client))
+
+
 static StackItem*
 stack_new(Client *client)
 {
@@ -50,6 +53,37 @@ stack_new(Client *client)
   return list;
 }
 
+static StackItem*
+stack_last()
+{
+  StackItem* list = comp_stack;
+
+  if (list == NULL) return NULL;
+
+  while (list->next != NULL) list = list->next;
+
+  return list;
+}
+
+static StackItem*
+stack_prev(Client *client)
+{
+  StackItem* cur = NULL, *ret = NULL;
+
+  if ((cur = comp_stack) == NULL || cur->client == client)
+    return NULL;
+  
+  while (cur)
+    {
+      if (cur->next && cur->next->client == client)
+	return cur;
+
+      cur = cur->next;
+    }
+
+  return NULL;
+}
+
 static void
 stack_push(Client *client)
 {
@@ -62,7 +96,8 @@ stack_push(Client *client)
     }
 
   new          = stack_new(client);
-  new->next    = comp_stack;
+  new->next        = comp_stack;
+
   comp_stack = new;
 }
 
@@ -559,6 +594,42 @@ client_border_size (Wm *w, Client *c, int x, int y)
     return border;
 }
 
+static Visual*
+comp_engine_get_argb32_visual(Wm *w)
+{
+  XVisualInfo		*xvi;
+  XVisualInfo		template;
+  int			nvi;
+  int			i;
+  XRenderPictFormat	*format;
+  Visual		*visual = NULL;
+
+  template.screen = w->screen;
+  template.depth  = 32;
+  template.class  = TrueColor;
+
+  if ((xvi = XGetVisualInfo (w->dpy, 
+			     VisualScreenMask |
+			     VisualDepthMask |
+			     VisualClassMask,
+			     &template,
+			     &nvi)) == NULL)
+    return NULL;
+  
+  for (i = 0; i < nvi; i++)
+    {
+      format = XRenderFindVisualFormat (w->dpy, xvi[i].visual);
+      if (format->type == PictTypeDirect && format->direct.alphaMask)
+	{
+	  visual = xvi[i].visual;
+	  break;
+	}
+    }
+
+  XFree (xvi);
+  return visual;
+}
+
 void
 comp_engine_set_defualts(Wm *w)
 {
@@ -580,7 +651,8 @@ comp_engine_set_defualts(Wm *w)
 void
 comp_engine_theme_init(Wm *w)
 {
-  Pixmap                        transPixmap, blackPixmap, lowlightPixmap;
+  Pixmap                        transPixmap, blackPixmap, lowlightPixmap, 
+                                redPixmap;
   XRenderPictureAttributes	pa;
   XRenderColor                  c;
   int                           i;
@@ -646,6 +718,22 @@ comp_engine_theme_init(Wm *w)
 
   XRenderFillRectangle (w->dpy, PictOpSrc, w->black_picture, &c, 0, 0, 1, 1);
 
+#if DEBUG
+  /* for visual composite debugging */
+
+  redPixmap = XCreatePixmap (w->dpy, w->root, 1, 1, 32);
+
+  w->red_picture 
+    = XRenderCreatePicture (w->dpy, redPixmap,
+			    XRenderFindStandardFormat (w->dpy, PictStandardARGB32),
+			    CPRepeat,
+			    &pa);
+
+  c.red = 0xffff; c.green = 0; c.blue = 0; c.alpha = 0xffff; 
+
+  XRenderFillRectangle (w->dpy, PictOpSrc, w->red_picture, &c, 0, 0, 1, 1);
+#endif
+
   /* Used for lowlights */
 
   lowlightPixmap = XCreatePixmap (w->dpy, w->root, 1, 1, 32);
@@ -750,6 +838,7 @@ comp_engine_init (Wm *w)
   int		                xfixes_event, xfixes_error;
   int		                render_event, render_error;
   XRenderPictureAttributes	pa;
+  Pixmap                        tmp_pxm;
 
   dbg("%s() called\n", __func__);
 
@@ -808,12 +897,29 @@ comp_engine_init (Wm *w)
 
   dbg("%s() success \n", __func__);
 
+  /* Setup an mbpixbuf reference for 32bit argb rendering */
+
+  w->argb_pb = mb_pixbuf_new_extended(w->dpy, 
+				      w->screen, 
+				      comp_engine_get_argb32_visual(w),
+				      32);
+
+  /* A little hack to get the GC right */
+
+  tmp_pxm = XCreatePixmap(w->dpy, w->root, 16, 16, 32);
+
+  w->argb_pb->gc = XCreateGC(w->dpy, tmp_pxm, 0, NULL);
+
+  XFreePixmap(w->dpy, tmp_pxm);
+
    return True;
 }
 
 void
 comp_engine_client_init(Wm *w, Client *client)
 {
+  XRenderPictFormat *format;
+
   if (!w->have_comp_engine) return;
 
   client->damaged      = 0;
@@ -823,6 +929,17 @@ comp_engine_client_init(Wm *w, Client *client)
   client->borderSize   = None;
   client->extents      = None;
   client->transparency = -1;
+  client->is_argb32    = False;
+
+  /* Check visual */
+
+  format = XRenderFindVisualFormat (w->dpy, client->visual);
+  
+  if (format && format->type == PictTypeDirect && format->direct.alphaMask)
+    {
+      dbg("%s() client is ARGB32\n", __func__);
+      client->is_argb32 = True;
+    }
 
   comp_engine_client_get_trans_prop(w, client);
 
@@ -892,15 +1009,25 @@ comp_engine_client_show(Wm *w, Client *client)
 void
 comp_engine_client_hide(Wm *w, Client *client)
 {
+  Client *t = NULL;
+
   if (!w->have_comp_engine) return;
 
   dbg("%s() called\n", __func__);
 
-  /* Make sure any lowlighting gets cleared - does opposite
   if (client->flags & CLIENT_IS_MODAL_FLAG
       && ((t = wm_get_visible_main_client(w)) != NULL))
-    comp_engine_client_repair (w, t);
-  */
+    {
+      /* We need to make sure the any lowlighting on a 'parent' 
+       * modal for app gets cleared. This is kind of a sledgehammer	 
+       * approach to it, but more suttle attempts oddly fail at times.
+       *
+       * FIXME: keep an eye on this for future revisions of composite
+       *        - there may be a better way.
+       */
+      comp_engine_client_repair (w, t); 
+      comp_engine_add_damage (w, t->extents); 
+    }
 
   if (client->damage != None)
     {
@@ -984,7 +1111,21 @@ comp_engine_client_repair (Wm *w, Client *client)
 void
 comp_engine_client_configure(Wm *w, Client *client)
 {
-  /* XXX not sure what to do here, if we even need anything */
+  XserverRegion   damage = None;
+
+  XserverRegion   extents = client_win_extents(w, client);
+
+  damage = XFixesCreateRegion (w->dpy, 0, 0);
+  if (client->extents != None)
+    XFixesCopyRegion (w->dpy, damage, client->extents);
+
+
+  XFixesUnionRegion (w->dpy, damage, damage, extents);
+  XFixesDestroyRegion (w->dpy, extents);
+
+  comp_engine_add_damage (w, damage);
+
+  stack_top(client);
 }
 
 
@@ -1039,16 +1180,19 @@ _render_a_client(Wm           *w,
 
   /* Transparency only done for dialogs and overides */
 
-  if ( client->transparency == -1 
+  if ( (client->transparency == -1  
        || client->type == mainwin 
        || client->type == desktop
        || client->type == toolbar
-       || client->type == dock)
+	|| client->type == dock) && !client->is_argb32) 
     {
       XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 0, 0, region);
+
       XFixesSubtractRegion (w->dpy, region, region, winborder);
 
-      XRenderComposite (w->dpy, PictOpSrc, client->picture, None, w->root_buffer,
+      XRenderComposite (w->dpy, PictOpSrc, 
+			client->picture, 
+			None, w->root_buffer,
 			0, 0, 0, 0, x, y, width, height);
 
     }
@@ -1092,9 +1236,6 @@ comp_engine_destroy_root_buffer(Wm *w)
     }
 }
 
-
-
-
 void
 comp_engine_render(Wm *w, XserverRegion region)
 {
@@ -1148,12 +1289,16 @@ comp_engine_render(Wm *w, XserverRegion region)
 
   XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 0, 0, region);
 
-  /*
-#if MONITOR_REPAINT
-  XRenderComposite (w->dpy, PictOpSrc, w->black_picture, None, w->root_picture,
-		    0, 0, 0, 0, 0, 0, w->dpy_width, w->dpy_height);
+#if DEBUG
+  if (w->flags & DEBUG_COMPOSITE_VISIBLE_FLAG)
+    {
+      XRenderComposite (w->dpy, PictOpSrc, w->red_picture, 
+			None, w->root_picture,
+			0, 0, 0, 0, 0, 0, w->dpy_width, w->dpy_height);
+      XSync(w->dpy, False);
+      /* return; */
+    }
 #endif
-  */
 
   /* Render top -> bottom */
 
@@ -1223,7 +1368,7 @@ comp_engine_render(Wm *w, XserverRegion region)
 
   /* Now render shadows */
 
-  stack_enumerate(item)
+  stack_enumerate_backwards(item)
     {
       t = item->client;
       if ((t->type == dialog 
@@ -1240,14 +1385,17 @@ comp_engine_render(Wm *w, XserverRegion region)
 	      Picture shadow_pic;
 	  
 	      t->get_coverage(t, &x, &y, &width, &height);  
+
 	  
-	      if (w->config->shadow_style == SHADOW_STYLE_SIMPLE)
+	      if (w->config->shadow_style == SHADOW_STYLE_SIMPLE) 
 		{
 		  XserverRegion shadow_region;
-		  
+
 		  /* Grab 'shape' region of window */
 		  shadow_region = client_border_size (w, t, x, y);
 		  
+
+		  
 		  /* Offset it. */
 		  XFixesTranslateRegion (w->dpy, shadow_region, 
 					 w->config->shadow_dx, 
@@ -1259,56 +1407,108 @@ comp_engine_render(Wm *w, XserverRegion region)
 		  
 		  XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 
 					      0, 0, shadow_region);
-		  
-		  XRenderComposite (w->dpy, PictOpOver, w->black_picture, 
-				    None, 
-				    w->root_buffer,
-				    0, 0, 0, 0,
-				    x + w->config->shadow_dx,
-				    y + w->config->shadow_dy,
-				    width  + w->config->shadow_padding_width, 
-				    height + w->config->shadow_padding_height);
 
-		  if (t->transparency != -1)
-		    XRenderComposite (w->dpy, PictOpOver, t->picture, w->trans_picture,
-				      w->root_buffer, 0, 0, 0, 0, x, y, width, height);
-		  
+		  /* shadows */
+		  if (t->is_argb32 )
+		    {
+		      XRenderComposite (w->dpy, PictOpOver, 
+					w->black_picture,
+					t->picture, 
+ 					w->root_buffer,
+					0, 0, 0, 0,
+					x + w->config->shadow_dx,
+					y + w->config->shadow_dy,
+					width  + w->config->shadow_padding_width, 
+					height + w->config->shadow_padding_height);
+
+		    }
+		  else
+		    {
+		      XRenderComposite (w->dpy, PictOpOver, 
+					w->black_picture, 
+					None, 
+					w->root_buffer,
+					0, 0, 0, 0,
+					x + w->config->shadow_dx,
+					y + w->config->shadow_dy,
+					width  + w->config->shadow_padding_width, 
+					height + w->config->shadow_padding_height);
+		    }
+
+		  /* Paint any trans window contents */
+		  if (t->transparency != -1 || t->is_argb32 )
+		    {
+		      XFixesDestroyRegion (w->dpy, shadow_region);
+
+		      shadow_region = client_border_size (w, t, x, y);
+
+		      XFixesIntersectRegion (w->dpy, shadow_region,
+					     t->border_clip, shadow_region );
+		      
+		      XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 
+						  0, 0, shadow_region);
+
+		      if (t->is_argb32)
+			XRenderComposite (w->dpy, PictOpOver, 
+					  t->picture, None,
+					  w->root_buffer, 0, 0, 0, 0, 
+					  x, y, width, height);
+		      else
+			XRenderComposite (w->dpy, PictOpOver, 
+					  t->picture, w->trans_picture,
+					  w->root_buffer, 0, 0, 0, 0, 
+					  x, y, width, height);
+
+		    }
+
 		  XFixesDestroyRegion (w->dpy, shadow_region);
 		}
 	      else 		/* GAUSSIAN */
 		{
 		  
-		  /* Combine pregenerated shadow tiles */
-		  shadow_pic 
-		    = shadow_gaussian_make_picture (w, 
-						    width + w->config->shadow_padding_width, 
-						    height + w->config->shadow_padding_height);
-		  
 		  XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 
 					      0, 0, t->border_clip);
-		  
-		  XRenderComposite (w->dpy, PictOpOver, w->black_picture, 
-				    shadow_pic, 
-				    w->root_buffer,
-				    0, 0, 0, 0,
-				    x + w->config->shadow_dx,
-				    y + w->config->shadow_dy,
-				    width + w->config->shadow_padding_width, 
-				    height + w->config->shadow_padding_height);
-		  
-		  if (t->transparency != -1)
-		    XRenderComposite (w->dpy, PictOpOver, t->picture, w->trans_picture,
-				      w->root_buffer, 0, 0, 0, 0, x, y, width, height);
 
+		  if (t->transparency != -1 || t->is_argb32 )
+		    {
+		      /* No shadows currently for transparent windows */
+		      XRenderComposite (w->dpy, PictOpOver, 
+					t->picture, w->trans_picture,
+					w->root_buffer, 0, 0, 0, 0, 
+					x, y, width, height);
+		      /*
+		      XRenderComposite (w->dpy, PictOpOver, 
+					t->picture, None,
+					w->root_buffer, 0, 0, 0, 0, 
+					x, y, width, height);
+		      */
 
-		  XRenderFreePicture (w->dpy, shadow_pic);
-		  
+		    } else {		  
+		      /* Combine pregenerated shadow tiles */
+
+		      shadow_pic 
+			= shadow_gaussian_make_picture (w, 
+							width + w->config->shadow_padding_width, 
+							height + w->config->shadow_padding_height);
+
+		      XRenderComposite (w->dpy, PictOpOver, w->black_picture, 
+					shadow_pic, 
+					w->root_buffer,
+					0, 0, 0, 0,
+					x + w->config->shadow_dx,
+					y + w->config->shadow_dy,
+					width + w->config->shadow_padding_width, 
+					height + w->config->shadow_padding_height);
+		      XRenderFreePicture (w->dpy, shadow_pic);
+
+		    }
 		}
 	      
 	    }
 
 	}
     }
+
   
   XFixesSetPictureClipRegion (w->dpy, w->root_buffer, 0, 0, None);
 
@@ -1323,7 +1523,6 @@ comp_engine_render(Wm *w, XserverRegion region)
 void
 comp_engine_time(Wm *w)
 {
-
 #ifdef STANDALONE   	/* No timings for standalone */
   return;
 #endif /* STANDALONE */
@@ -1437,5 +1636,9 @@ comp_engine_time(Wm *w)
 
 #endif /* USE_COMPOSITE */
 
-#endif
+#else
+
+  fprintf(stderr, "matchbox-window-manager: timing functionality disabled\n");
+
+#endif /* DO TIMINGS */
 }
